@@ -1,6 +1,6 @@
 /*
  *  callback.c, gKamus (http://gkamus.sourceforge.net)
- *  Copyright (C) 2008-2009, Ardhan Madras <ajhwb@knac.com>
+ *  Copyright (C) 2008-2011, Ardhan Madras <ajhwb@knac.com>
  *
  *  gKamus is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,8 +33,9 @@
 #include "function.h"
 
 static void modified_notify (gboolean);
-static void change_cursor (void);
+static gboolean change_cursor (gpointer);
 static gboolean restore_cursor (gpointer);
+static gboolean modify_treev (gpointer);
 static gboolean set_sensitive_timer (GtkWidget*);
 static gboolean change_dict (gpointer);
 static gboolean modified = FALSE;
@@ -88,19 +89,21 @@ set_sensitive_timer (GtkWidget *widget)
 /*
  * Ganti cursor ke mode "busy".
  */
-static void
-change_cursor (void)
+static gboolean
+change_cursor (gpointer data)
 {
   GdkScreen *screen;
   GdkWindow *window;
   GdkCursor *cursor;
 
   screen = gtk_window_get_screen ((GtkWindow *) window_main);
-  window = gdk_screen_get_root_window (screen);
+  window = gdk_screen_get_active_window (screen);
   cursor = gdk_cursor_new (GDK_WATCH); /* lihat gdkcursor.h */
 
   gdk_window_set_cursor (window, cursor);
+  g_object_unref (window);
   gdk_cursor_unref (cursor);
+  return FALSE;
 }
 
 /*
@@ -114,10 +117,11 @@ restore_cursor (gpointer data)
   GdkCursor *cursor;
 
   screen = gtk_window_get_screen ((GtkWindow *) window_main);
-  window = gdk_screen_get_root_window (screen);
+  window = gdk_screen_get_active_window (screen);
   cursor = gdk_cursor_new (GDK_LEFT_PTR);
 
   gdk_window_set_cursor (window, cursor);
+  g_object_unref (window);
   gdk_cursor_unref (cursor);
   return FALSE;
 }
@@ -129,8 +133,9 @@ on_window_main_delete_event (GtkWidget *widget,
 {
   if (modified)
     {
-      gint d = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Keluar",
-                              "Kamus belum disimpan, keluar sekarang juga?");
+      gint d = create_dialog (GTK_MESSAGE_QUESTION, window_main, 
+                              "Konfirmasi Keluar", "Kamus belum disimpan, "
+                              "keluar sekarang juga?");
       if (d != GTK_RESPONSE_OK)
         return TRUE; /* propagate */
     }
@@ -138,6 +143,15 @@ on_window_main_delete_event (GtkWidget *widget,
   return TRUE; /* agar window_main tdk di destroy */
 }
 
+void
+on_entry_search_icon_pressed (GtkWidget            *entry,
+                              GtkEntryIconPosition  pos,
+                              GdkEvent             *event,
+                              gpointer              data)
+{
+  gtk_entry_set_text ((GtkEntry *) entry, "");
+}
+
 /*
  * Cari kata yang paling mendekati entry.
  */
@@ -148,6 +162,9 @@ on_entry_search_changed (GtkWidget *entry,
   gchar *word;
   gint len;
 
+  if (!gtk_check_menu_item_get_active ((GtkCheckMenuItem *) menu_autosearch))
+    return;
+  
   word = g_strdup (gtk_entry_get_text ((GtkEntry *) entry));
   word = g_strstrip (word);
   len = strlen (word);
@@ -182,6 +199,16 @@ on_window_modify_destroy (GtkWidget *widget,
   gtk_widget_destroy (window_modify);
 }
 
+static gboolean
+modify_treev (gpointer data)
+{
+  set_treev_word ();
+  g_idle_add ((GSourceFunc) restore_cursor, NULL);
+  g_idle_add ((GSourceFunc) set_sensitive_timer, window_main);
+  select_treev_word (GPOINTER_TO_INT (data));
+  return FALSE;
+}
+
 /*
  * Tambah / Edit entry.
  */
@@ -199,7 +226,7 @@ on_button_mod_ok_clicked (GtkWidget *widget,
 
   tmp = g_ascii_strdown (gtk_entry_get_text ((GtkEntry *) entry_mod), -1);
   tmp = g_strstrip (tmp);
-  g_snprintf (word, sizeof(word), tmp);
+  g_snprintf (word, sizeof(word), "%s", tmp);
   g_free (tmp);
 
   buf = gtk_text_view_get_buffer ((GtkTextView *) textv_mod);
@@ -209,7 +236,7 @@ on_button_mod_ok_clicked (GtkWidget *widget,
   tmp = g_strstrip (tmp);
   /* lihat on_textv_mod_key_press_event() */
   gtk_text_buffer_set_text (buf, tmp, -1);
-  g_snprintf (def, sizeof(def), tmp);
+  g_snprintf (def, sizeof(def), "%s", tmp);
   g_free (tmp);
 
   /* cek input */
@@ -234,7 +261,7 @@ on_button_mod_ok_clicked (GtkWidget *widget,
   GtkTreeModel *model;
   GtkTreePath *path;
   GtkTreeIter iter;
-  gint *id;
+  gint *id, new_index, old_index;
 
   /* cari data list sesuai indeks treeview */
   selection = gtk_tree_view_get_selection ((GtkTreeView *) treev_word);
@@ -255,9 +282,10 @@ on_button_mod_ok_clicked (GtkWidget *widget,
     {
       create_dialog (GTK_MESSAGE_WARNING, window_modify, "Edit Kata",
                      "Kata \"%s\" telah ada dalam database kamus.", word);
-     return;
+      return;
     }
 
+  gtk_widget_destroy (window_modify);
   if (i == GKAMUS_MODIFY_ADD)
     {
       Dictionary *new_dic;
@@ -268,13 +296,15 @@ on_button_mod_ok_clicked (GtkWidget *widget,
       new_dic->definition = g_strdup (def);
       dict = g_list_prepend (dict, new_dic);
       /* re-sort @dict */
-      set_sensitive (window_main, FALSE);
       dict = g_list_sort (dict, (GCompareFunc) list_compare);
-      set_treev_word ();
-      select_treev_word (g_list_index (dict, new_dic));
-      g_idle_add ((GSourceFunc) set_sensitive_timer, window_main);
+      new_index = g_list_index (dict, new_dic);
+      set_sensitive (window_main, FALSE);
+      /* beri waktu sampai window_modify "unrealized", agar cursor 
+         terganti dengan benar */
+      g_timeout_add (50, (GSourceFunc) change_cursor, NULL);
+      g_timeout_add (100, (GSourceFunc) modify_treev, GINT_TO_POINTER (new_index));
       g_print ("added: %s\nindices = %i\ntotal: %i\n\n", new_dic->word,
-               g_list_index (dict, new_dic), g_list_length (dict));
+               new_index, g_list_length (dict));
     }
   else if (i == GKAMUS_MODIFY_EDIT)
     {
@@ -283,15 +313,16 @@ on_button_mod_ok_clicked (GtkWidget *widget,
       g_free (old_dic->definition);
       old_dic->word = g_strdup (word);
       old_dic->definition = g_strdup (def);
+      old_index = g_list_index (dict, old_dic);
 
       /* jika kata berubah, sort list dan load kembali */
       if (strcmp (old_dic->word, tmp))
         {
           dict = g_list_sort (dict, (GCompareFunc) list_compare);
+          new_index = g_list_index (dict, old_dic);
           set_sensitive (window_main, FALSE);
-          set_treev_word ();
-          g_idle_add ((GSourceFunc) set_sensitive_timer, window_main);
-          select_treev_word (g_list_index (dict, old_dic));
+          g_timeout_add (50, (GSourceFunc) change_cursor, NULL);
+          g_timeout_add (100, (GSourceFunc) modify_treev, GINT_TO_POINTER (new_index));
         }
       else
         {
@@ -299,15 +330,14 @@ on_button_mod_ok_clicked (GtkWidget *widget,
           gtk_text_buffer_set_text (buf, old_dic->definition, -1);
         }
       g_print ("edited: %s -> %s\nindices = %i\ntotal: %i\n\n", tmp, old_dic->word,
-               g_list_index (dict, old_dic), g_list_length (dict));
+               old_index, g_list_length (dict));
       g_free (tmp);
     }
   modified_notify (TRUE);
-  gtk_widget_destroy (window_modify);
 }
 
 /*
- * Signal "key-press-event textv_mod.
+ * Signal "key-press-event" textv_mod.
  */
 gboolean
 on_textv_mod_key_press_event (GtkWidget   *widget,
@@ -412,7 +442,7 @@ on_menu_saveas_activate (GtkWidget *widget,
     {
       gtk_widget_destroy (dialog);
       gint r = create_dialog (GTK_MESSAGE_QUESTION, window_main, 
-                              "Simpan Kamus", "File \"%s\" telah ada, tulis "
+                              "Simpan kamus", "File \"%s\" telah ada, tulis "
                               "ulang file ini?", file);
       if (r != GTK_RESPONSE_OK)
         {
@@ -421,11 +451,8 @@ on_menu_saveas_activate (GtkWidget *widget,
         }
     }
   if (!write_dict (file))
-    {
-     gtk_widget_destroy (dialog);
-     create_dialog (GTK_MESSAGE_ERROR, window_main, "Simpan Kamus",
+     create_dialog (GTK_MESSAGE_ERROR, window_main, "Simpan kamus",
                     "Kamus tidak dapat disimpan, periksa permisi anda.");
-    }
   g_free (file);
   gtk_widget_destroy (dialog);
 }
@@ -454,8 +481,8 @@ on_menu_copy_activate (GtkWidget *widget,
   selection = gtk_tree_view_get_selection ((GtkTreeView *) treev_word);
   if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-      create_dialog (GTK_MESSAGE_WARNING, window_main, "Copy Kata",
-                     "Silahkan pilih kata yang akan di copy.\n");
+      create_dialog (GTK_MESSAGE_WARNING, window_main, "Salin Kata",
+                     "Silahkan pilih kata yang akan disalin.\n");
       return;
     }
 
@@ -510,7 +537,7 @@ on_menu_edit_activate (GtkWidget *widget,
   selected = gtk_tree_selection_get_selected (selection, &model, &iter);
   if (!selected)
     {
-      create_dialog (GTK_MESSAGE_WARNING, window_main, "Edit Kata",
+      create_dialog (GTK_MESSAGE_WARNING, window_main, "Edit kata",
                      "Silahkan pilih kata yang akan diedit.");
       return;
     }
@@ -550,7 +577,7 @@ on_menu_delete_activate (GtkWidget *widget,
   selected = gtk_tree_selection_get_selected (selection, &model, &iter);
   if (!selected)
     {
-      create_dialog (GTK_MESSAGE_WARNING, window_main, "Hapus Kata",
+      create_dialog (GTK_MESSAGE_WARNING, window_main, "Hapus kata",
                      "Silahkan pilih kata yang akan dihapus.");
       return;
     }
@@ -564,7 +591,7 @@ on_menu_delete_activate (GtkWidget *widget,
   dic = g_list_nth_data (dict, *id);
 
   ret = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Hapus Kata", "Hapus "
-                       "entri kata \"%s\" dari database kamus?", dic->word);
+                       "entri kata \"%s\" dari kamus?", dic->word);
   if (ret != GTK_RESPONSE_OK)
     {
       gtk_tree_path_free (path);
@@ -641,7 +668,7 @@ on_menu_tool_ei_activate (GtkWidget *widget,
         on_menu_save_activate (NULL, NULL);
     }
   set_sensitive (window_main, FALSE);
-  change_cursor ();
+  g_timeout_add (50, (GSourceFunc) change_cursor, NULL);
   g_timeout_add (100, (GSourceFunc) change_dict, GINT_TO_POINTER (GKAMUS_DIC_EN));
   modified_notify (FALSE);
 }
@@ -660,13 +687,13 @@ on_menu_tool_ie_activate (GtkWidget *widget,
     {
       gint ret;
       
-      ret = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Ganti Kamus",
+      ret = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Ganti kamus",
                            "Simpan kamus sebelum mengganti model kamus?");
       if (ret == GTK_RESPONSE_OK)
         on_menu_save_activate (NULL, NULL);
     }
   set_sensitive (window_main, FALSE);
-  change_cursor ();
+  g_timeout_add (50, (GSourceFunc) change_cursor, NULL);
   g_timeout_add (100, (GSourceFunc) change_dict, GINT_TO_POINTER (GKAMUS_DIC_ID));
   modified_notify (FALSE);
 }
@@ -678,8 +705,6 @@ void
 on_menu_alpha_activate (GtkWidget *widget, 
                         gpointer   data)
 {
-  if (!window_alpha)
-    window_alpha = create_window_alpha ();
   gtk_window_present ((GtkWindow *) window_alpha);
 }
 
@@ -701,19 +726,19 @@ on_menu_verb_activate (GtkWidget *widget,
   const gchar verbs[] = "irregular-verbs";
 
 #ifdef PACKAGE_DATA_DIR
-  file = g_build_filename (PACKAGE_DATA_DIR, "gkamus", verbs, NULL);
+  file = g_build_filename (PACKAGE_DATA_DIR, verbs, NULL);
 #else
   file = g_build_filename ("share", "data", verbs, NULL);
 #endif
   read = g_io_channel_new_file (file, "r", NULL);
-  g_free (file);
+
   if (!read)
     {
-      g_warning ("File irregular verbs tidak dapat dibuka.\n");
-      create_dialog (GTK_MESSAGE_ERROR, window_main, "Error", "File irreguler "
-                     "verbs tidak dapat dibuka.");
+      g_warning ("could not read file %s\n", file);
+      g_free (file);
       return;
     }
+  g_free (file);
 
   GtkTreeModel *model;
   GtkTreeIter iter;
@@ -736,49 +761,20 @@ on_menu_verb_activate (GtkWidget *widget,
       g_free (line);
       g_strfreev (split);
     }
-  g_io_channel_shutdown (read, TRUE, NULL);
+  g_io_channel_shutdown (read, FALSE, NULL);
   g_io_channel_unref (read);
   gtk_widget_show (window_verb);
 }
 
-#ifdef G_OS_UNIX
-/* 
- * Menu Bantuan -> Buat shortcut -> Desktop. 
- */
 void
-on_menu_short_desktop_activate (GtkWidget *widget, 
-                                gpointer   data)
+on_menu_tenses_activate (GtkWidget *widget,
+                         gpointer   data)
 {
-  gint d = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Buat Shortcut",
-                          "gKamus akan membuat shortcut di Desktop, ini "
-                          "mungkin tidak bekerja di sistem anda.");
-  if (d != GTK_RESPONSE_OK)
-    return;
-  if (generate_shortcut (GKAMUS_SHORTCUT_DESKTOP) == -1)
-    create_dialog (GTK_MESSAGE_WARNING, window_main, "Buat Shortcut",
-                   "gKamus tidak dapat membuat shortcut di Desktop, "
-                   "periksa permisi anda.");
+  if (!window_tenses)
+    window_tenses = create_window_tenses ();
+  gtk_window_present ((GtkWindow *) window_tenses);
 }
 
-/* 
- * Menu Bantuan -> Buat shortcut -> Menu.
- */
-void
-on_menu_short_menu_activate (GtkWidget *widget, 
-                             gpointer   data)
-{
-  gint d = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Buat Shortcut",
-                          "gKamus akan membuat shortcut di Menu, ini "
-                          "mungkin tidak bekerja di sistem anda.");
-  if (d != GTK_RESPONSE_OK)
-    return;
-  if (generate_shortcut (GKAMUS_SHORTCUT_MENU) == -1)
-    create_dialog (GTK_MESSAGE_WARNING, window_main, "Buat Shortcut",
-                   "gKamus tidak dapat membuat shortcut di Menu, "
-                   "periksa permisi anda.");
-}
-#endif
-
 /*
  * Menu Bantuan -> Tentang. 
  */
@@ -791,6 +787,121 @@ on_menu_about_activate (GtkWidget *widget,
     gtk_widget_destroy (widget);
 }
 
+typedef struct
+{
+  gsize pos;
+  gint id;
+} TagData;
+
+gint
+comp_func (const TagData *a,
+           const TagData *b)
+{
+  return a->pos - b->pos;
+}
+
+/*
+ * Highlight keterangan-keterangan tambahan pada definisi.
+ */
+static void
+insert_with_id_tag (const gchar *word,
+                    const gchar *text)
+{
+  g_return_if_fail (text != NULL && word != NULL);
+
+  GtkTextBuffer *buffer;
+  static GtkTextTag *word_tag = NULL;
+  static GtkTextTag *tag = NULL;
+  GtkTextIter iter;
+  GList *list = NULL, *ptr;
+  gchar *tmp, *definition, *kb;
+  gchar **info = NULL;
+  gint id;
+
+  static const gchar * const eng_info[] = 
+  { 
+    "lih", "kk.", "ks.", "kb.", "kd.", "kg.", "kkt.", "kki.", NULL 
+  };
+
+  static const gchar * const id_info[] = 
+  { 
+    "see ", "k.o.", NULL 
+  };
+
+  buffer = gtk_text_view_get_buffer ((GtkTextView *) textv_definition);
+  definition = g_strdup (text);
+  info = g_strdupv ((gchar **) (eng ? eng_info : id_info));
+
+  for (id = 0; info[id]; id++)
+    {
+      tmp = definition;
+      while ((kb = strstr (tmp, info[id])))
+        {
+          TagData *pt = g_new (TagData, 1);
+
+          pt->id = id;
+          pt->pos = kb - definition;
+          list = g_list_prepend (list, pt);
+          tmp = kb + strlen (info[id]);
+        }
+    }
+
+  gtk_text_buffer_set_text (buffer, "", 0);
+  gtk_text_buffer_get_start_iter (buffer, &iter);
+  if (!word_tag)
+    word_tag = gtk_text_buffer_create_tag (buffer, NULL, 
+                                           "scale", PANGO_SCALE_LARGE, 
+                                           "weight", PANGO_WEIGHT_BOLD,
+                                           NULL);
+  gtk_text_buffer_insert_with_tags (buffer, &iter, word, -1, word_tag, NULL);
+  gtk_text_buffer_insert (buffer, &iter, "\n", -1);
+
+  if (list == NULL)
+    {
+      gtk_text_buffer_insert (buffer, &iter, definition, -1);
+      g_free (definition);
+      g_strfreev (info);
+      return;
+    }
+
+  list = g_list_sort (list, (GCompareFunc) comp_func);
+  ptr = list;
+  tmp = definition;
+
+  while (*tmp)
+    {
+      TagData *pt;
+      gpointer pos = 0;
+
+      if (ptr)
+        {
+          pt = ptr->data;
+          pos = definition + pt->pos;
+        }
+
+      if (tmp == pos)
+        {
+          g_print ("pt->id: %i, pt->pos: %i\n", pt->id, pt->pos);
+          gsize len = strlen (info[pt->id]);
+          if (!tag)
+            tag = gtk_text_buffer_create_tag (buffer, NULL, "weight", PANGO_WEIGHT_BOLD, NULL);
+          gtk_text_buffer_insert_with_tags (buffer, &iter, tmp, len, tag, NULL);
+          tmp += len;
+          ptr = ptr->next;
+        }
+      else
+        {
+          gtk_text_buffer_insert (buffer, &iter, tmp, 1);
+          tmp++;
+        }
+    }
+
+  g_list_foreach (list, (GFunc) g_free, NULL);
+  g_list_free (list);
+  g_free (definition);
+  g_strfreev (info);
+}
+
 /* 
  * Ketika list kata di pilih. 
  */
@@ -803,23 +914,22 @@ on_treev_word_columns_changed (GtkWidget *widget,
   GtkTreeIter iter;
 
   selection = gtk_tree_view_get_selection ((GtkTreeView *) treev_word);
-  if (gtk_tree_selection_get_selected (selection, &model, &iter))
-    {
-      Dictionary *dic;
-      GtkTreePath *path;
-      GtkTextBuffer *buffer;
-      gint *id;
+  if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+    return;
 
-      buffer = gtk_text_view_get_buffer ((GtkTextView *) textv_definition);
-      path = gtk_tree_model_get_path (model, &iter);
-      id = gtk_tree_path_get_indices (path);
-      dic = g_list_nth_data (dict, *id);
+  Dictionary *dic;
+  GtkTreePath *path;
+  GtkTextBuffer *buffer;
+  gint *id;
 
-      gtk_text_buffer_set_text (buffer, dic->definition, -1);
-      g_print ("selected: %s\nindices: %i\ntotal: %i\n\n", dic->word, 
-               *id, g_list_length (dict));
-      gtk_tree_path_free (path);
-    }
+  buffer = gtk_text_view_get_buffer ((GtkTextView *) textv_definition);
+  path = gtk_tree_model_get_path (model, &iter);
+  id = gtk_tree_path_get_indices (path);
+  dic = g_list_nth_data (dict, *id);
+
+  insert_with_id_tag (dic->word, dic->definition);
+  g_print ("selected: %s\nindices: %i\n\n", dic->word, *id);
+  gtk_tree_path_free (path);
 }
 
 /* 
@@ -854,8 +964,9 @@ on_button_find_clicked (GtkWidget *widget,
 
   if (i == -1)
     {
+#if 0
       g_print ("search: %s [not found]\n", word);
-      i = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Pencarian",
+      i = create_dialog (GTK_MESSAGE_QUESTION, window_main, "Pencarian Kata",
                          "Kata yang dicari tidak ditemukan, tambah kata ini?");
       if (i == GTK_RESPONSE_OK)
         {
@@ -863,13 +974,14 @@ on_button_find_clicked (GtkWidget *widget,
           gtk_entry_set_text ((GtkEntry *) entry_mod, tmp);
           gtk_widget_show (window_modify);
         }
+#endif
     }
   else
     {
       g_timer_stop (timer);
       gdouble t = g_timer_elapsed (timer, NULL);
-      g_print ("search time: %f second(s).\n", t);
       select_treev_word (i);
+      g_print ("search time: %f second(s).\n", t);
     }
   g_timer_destroy (timer);
   g_free (tmp);
@@ -892,3 +1004,66 @@ on_window_verb_destroy (GtkWidget *widget,
   window_verb = NULL;
 }
 
+void
+on_window_tenses_destroy (GtkWidget *widget,
+                          gpointer   data)
+{
+  gtk_widget_destroy (window_tenses);
+  window_tenses = NULL;
+}
+
+/*
+ * Baca file tenses sesuai pilihan di combobox.
+ */
+void
+on_combob_tenses_changed (GtkComboBox *combob,
+                          gpointer     data)
+{
+  static const gchar * const tenses[] =
+  {
+    "simplepresent",
+    "presentcontinuous",
+    "simplepast",
+    "pastcontinuous",
+    "presentperfect",
+    "presentperfectcontinuous",
+    "pastperfect",
+    "pastperfectcontinuous",
+    "simplefuture",
+    "futurecontinuous",
+    "futureperfect",
+    "futureperfectcontinuous"
+  };
+  gchar *file, *tmp;
+  gint i;
+  GtkTextBuffer *buffer;
+  GIOStatus status;
+  GIOChannel *read;
+  GError *error = NULL;
+
+  i = gtk_combo_box_get_active (combob);
+#ifdef PACKAGE_DATA_DIR
+  file = g_build_filename (PACKAGE_DATA_DIR, tenses[i], NULL);
+#else
+  tmp = g_get_current_dir ();
+  file = g_build_filename (tmp, "share", "data", tenses[i], NULL);
+  g_free (tmp);
+#endif
+  read = g_io_channel_new_file (file, "r", &error);
+  if (!read)
+    {
+      g_warning ("%s: %s: %s\n", __func__, tenses[i], error->message);
+      g_free (file);
+      g_error_free (error);
+      return;
+    }
+  buffer = gtk_text_view_get_buffer ((GtkTextView *) textv_tenses);
+  status = g_io_channel_read_to_end (read, &tmp, NULL, NULL);
+  if (status == G_IO_STATUS_NORMAL)
+    gtk_text_buffer_set_text (buffer, g_strstrip (tmp), -1);
+  g_io_channel_shutdown (read, TRUE, NULL);
+  g_io_channel_unref (read);
+  g_free (tmp);
+  g_free (file);
+}
+
