@@ -1,6 +1,6 @@
 /*
  *  gui.c, gKamus (http://gkamus.sourceforge.net)
- *  Copyright (C) 2008-2009, Ardhan Madras <ajhwb@knac.com>
+ *  Copyright (C) 2008-2011, Ardhan Madras <ajhwb@knac.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,13 +30,13 @@
 #include "callback.h"
 #include "function.h"
 
-#include "sexy-icon-entry.h"
 
 #define UI_FILE "gkamus.ui"
 
 /* window_main */
 GtkWidget *window_main = NULL;
 GtkWidget *menu_save = NULL;
+GtkWidget *menu_autosearch = NULL;
 GtkWidget *menu_tool_ei = NULL;
 GtkWidget *menu_tool_ie = NULL;
 GtkWidget *entry_search = NULL;
@@ -51,6 +51,10 @@ GtkWidget *window_alpha = NULL;
 /* window_verb */
 GtkWidget *window_verb = NULL;
 GtkWidget *treev_verb = NULL;
+/* window_tenses */
+GtkWidget *window_tenses = NULL;
+GtkWidget *combob_tenses = NULL;
+GtkWidget *textv_tenses = NULL;
 
 static GtkBuilder *builder = NULL;
 
@@ -72,9 +76,11 @@ builder_init (void)
   guint res;
 
 #ifdef PACKAGE_DATA_DIR
-  ui = g_build_filename (PACKAGE_DATA_DIR, "gkamus", UI_FILE, NULL);
+  ui = g_build_filename (PACKAGE_DATA_DIR, UI_FILE, NULL);
 #else
-  ui = g_build_filename (g_get_current_dir (), "share", "data", UI_FILE, NULL);
+  gchar *tmp = g_get_current_dir ();
+  ui = g_build_filename (tmp, "share", "data", UI_FILE, NULL);
+  g_free (tmp);
 #endif
   test = g_file_test (ui, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_EXISTS);
   if (!test)
@@ -211,9 +217,10 @@ create_dialog (GtkMessageType  type,
     va_end (args);
     dialog = gtk_message_dialog_new ((GtkWindow *) parent,
                                      GTK_DIALOG_MODAL, type, GTK_BUTTONS_NONE,
-                                     head != NULL ? head : tmp);
+                                     "%s", (head != NULL ? head : tmp));
     if (head)
-        gtk_message_dialog_format_secondary_text ((GtkMessageDialog *) dialog, tmp);
+        gtk_message_dialog_format_secondary_text 
+            ((GtkMessageDialog *) dialog, "%s", tmp);
     switch (type)
     {
         case GTK_MESSAGE_QUESTION:
@@ -254,16 +261,12 @@ create_window_main (void)
   builder_init ();
 
   GtkWidget *window;
-  GtkWidget *menu_saveas;
   GtkWidget *menu_quit;
   GtkWidget *menu_copy;
   GtkWidget *menu_paste;
-  GtkWidget *menu_add;
-  GtkWidget *menu_edit;
-  GtkWidget *menu_delete;
   GtkWidget *menu_alpha;
   GtkWidget *menu_verb;
-  GtkWidget *menu_short;
+  GtkWidget *menu_tenses;
   GtkWidget *menu_about;
   GtkWidget *label_search;
   GtkWidget *vbox_search;
@@ -276,9 +279,11 @@ create_window_main (void)
 
   window = get_widget ("window_main");
   set_icon ((GtkWindow *) window);
+  gtk_window_set_title ((GtkWindow *) window, PROGRAM_NAME);
   g_signal_connect ((GObject *) window, "delete-event",
                     (GCallback) on_window_main_delete_event, NULL);
 
+#if 0
   menu_save = get_widget ("menu_save");
   set_sensitive (menu_save, FALSE);
   g_signal_connect ((GObject *) menu_save, "activate",
@@ -287,7 +292,8 @@ create_window_main (void)
   menu_saveas = get_widget ("menu_saveas");
   g_signal_connect ((GObject *) menu_saveas, "activate",
                     (GCallback) on_menu_saveas_activate, NULL);
-  
+#endif
+
   menu_quit = get_widget ("menu_quit");
   g_signal_connect ((GObject *) menu_quit, "activate",
                     (GCallback) on_menu_quit_activate, NULL);
@@ -299,7 +305,8 @@ create_window_main (void)
   menu_paste = get_widget ("menu_paste");
   g_signal_connect ((GObject *) menu_paste, "activate",
                     (GCallback) on_button_paste_clicked, NULL);
-  
+
+#if 0
   menu_add = get_widget ("menu_add");
   g_signal_connect ((GObject *) menu_add, "activate",
                     (GCallback) on_menu_add_activate, NULL);
@@ -311,7 +318,8 @@ create_window_main (void)
   menu_delete = get_widget ("menu_delete");
   g_signal_connect ((GObject *) menu_delete, "activate",
                     (GCallback) on_menu_delete_activate, NULL);
-  
+#endif
+
   menu_tool_ei = get_widget ("menu_tool_ei");
   g_signal_connect ((GObject *) menu_tool_ei, "activate",
                     (GCallback) on_menu_tool_ei_activate, NULL);
@@ -319,6 +327,8 @@ create_window_main (void)
   menu_tool_ie = get_widget ("menu_tool_ie");
   g_signal_connect ((GObject *) menu_tool_ie, "activate",
                     (GCallback) on_menu_tool_ie_activate, NULL);
+                    
+  menu_autosearch = get_widget ("menu_autosearch");
   
   menu_alpha = get_widget ("menu_alpha");
   g_signal_connect ((GObject *) menu_alpha, "activate",
@@ -328,6 +338,11 @@ create_window_main (void)
   g_signal_connect ((GObject *) menu_verb, "activate",
                     (GCallback) on_menu_verb_activate, NULL);
 
+  menu_tenses = get_widget ("menu_tenses");
+  g_signal_connect ((GObject *) menu_tenses, "activate",
+                    (GCallback) on_menu_tenses_activate, NULL);
+
+#if 0
   menu_short = get_widget ("menu_short");
 
 #ifdef G_OS_UNIX /* Shortcut hanya untuk UNIX */
@@ -343,6 +358,7 @@ create_window_main (void)
                     (GCallback) on_menu_short_menu_activate, NULL);
 #else
   g_object_set (menu_short, "visible", FALSE, NULL);
+#endif
 #endif
 
   menu_about = get_widget ("menu_about");
@@ -351,13 +367,15 @@ create_window_main (void)
   
   vbox_search = get_widget ("vbox_search");
 
-  entry_search = sexy_icon_entry_new ();
-  sexy_icon_entry_add_clear_button ((SexyIconEntry *) entry_search);
-  sexy_icon_entry_set_icon_highlight ((SexyIconEntry *) entry_search,
-                                      SEXY_ICON_ENTRY_SECONDARY, TRUE);
+  entry_search = gtk_entry_new ();
+  gtk_entry_set_icon_from_stock ((GtkEntry *) entry_search,
+                                 GTK_ENTRY_ICON_SECONDARY,
+                                 GTK_STOCK_CLEAR);
   gtk_entry_set_activates_default ((GtkEntry *) entry_search, TRUE);
   gtk_entry_set_max_length ((GtkEntry *) entry_search, 20);
   gtk_container_add ((GtkContainer *) vbox_search, entry_search);
+  g_signal_connect ((GObject *) entry_search, "icon-press",
+                    (GCallback) on_entry_search_icon_pressed, NULL);
   g_signal_connect ((GObject *) entry_search, "changed",
                     (GCallback) on_entry_search_changed, NULL);
 
@@ -374,7 +392,7 @@ create_window_main (void)
 
   store = gtk_list_store_new (COLUMN_N, G_TYPE_STRING);
   renderer = gtk_cell_renderer_text_new ();
-  column = gtk_tree_view_column_new_with_attributes ("Kata", renderer,
+  column = gtk_tree_view_column_new_with_attributes ("Word", renderer,
                                                      "text", COLUMN_WORD, NULL);
   
   treev_word = get_widget ("treev_word");
@@ -458,7 +476,7 @@ create_window_alpha (void)
   GtkWidget *textv;
   GtkTextBuffer *buf;
   
-  gchar alpha[] =
+  const gchar alpha[] =
   (
     "A  \t  - \t  el   \t\t\t  N  \t  - \t  en     \n"
     "B  \t  - \t  bi:  \t\t\t  O  \t  - \t  ou     \n"
@@ -472,13 +490,14 @@ create_window_alpha (void)
     "J  \t  - \t  jei  \t\t\t  W  \t  - \t  dablyu \n"
     "K  \t  - \t  kei  \t\t\t  X  \t  - \t  eks    \n"
     "L  \t  - \t  el   \t\t\t  Y  \t  - \t  wai    \n"
-    "M  \t  - \t  em   \t\t\t  Z  \t  - \t  sed    \0"
+    "M  \t  - \t  em   \t\t\t  Z  \t  - \t  sed      "
   );
   
   gint i = 0, j = 0;
   gchar tmp[1024]; /* so you can clearly read the alpha string ;[ */
 
   window = get_widget ("window_alpha");
+  gtk_window_set_title ((GtkWindow *) window, "English Alphabet List");
   set_icon ((GtkWindow *) window);
   g_signal_connect ((GObject *) window, "delete-event", 
                     (GCallback) on_window_alpha_delete_event, NULL);
@@ -522,6 +541,7 @@ create_window_verb (void)
   GtkTreeViewColumn *column_participle;
   
   window = get_widget ("window_verb");
+  gtk_window_set_title ((GtkWindow *) window, "Irregular Verbs");
   set_icon ((GtkWindow *) window);
   g_signal_connect ((GObject *) window, "destroy",
                     (GCallback) on_window_verb_destroy, NULL);
@@ -531,18 +551,21 @@ create_window_verb (void)
   render_present = gtk_cell_renderer_text_new ();
   column_present = gtk_tree_view_column_new_with_attributes ("Present Tense",
                         render_present, "text", 0, NULL);
+  gtk_tree_view_column_set_expand (column_present, TRUE);
   gtk_tree_view_column_set_cell_data_func (column_present, render_present,
         (GtkTreeCellDataFunc) cell_data_func, NULL, NULL);
       
   render_past = gtk_cell_renderer_text_new ();
   column_past = gtk_tree_view_column_new_with_attributes ("Past Tense",
                     render_past, "text", 1, NULL);
+  gtk_tree_view_column_set_expand (column_past, TRUE);
   gtk_tree_view_column_set_cell_data_func (column_past, render_past,
         (GtkTreeCellDataFunc) cell_data_func, NULL, NULL);
 
   render_participle = gtk_cell_renderer_text_new ();
   column_participle = gtk_tree_view_column_new_with_attributes ("Past Participle",
                           render_participle, "text", 2, NULL);
+  gtk_tree_view_column_set_expand (column_participle, TRUE);
   gtk_tree_view_column_set_cell_data_func (column_participle, render_participle,
         (GtkTreeCellDataFunc) cell_data_func, NULL, NULL);
   
@@ -561,6 +584,63 @@ create_window_verb (void)
   return window;
 }
 
+/*
+ * Buat window_tenses.
+ */
+GtkWidget*
+create_window_tenses (void)
+{
+  builder_init ();
+
+  GtkWidget *window;
+  GtkWidget *button_tenses;
+  GtkTreeModel *model;
+  
+  gint i = 0;
+  static const gchar * const tenses[] =
+  {
+    "Simple Present",
+    "Present Continuous",
+    "Simple Past",
+    "Past Continuous",
+    "Present Perfect",
+    "Present Perfect Continuous",
+    "Past Perfect",
+    "Past Perfect Continuous",
+    "Simple Future",
+    "Future Continuous",
+    "Future Perfect",
+    "Future Perfect Continuous"
+  };
+  
+  window = get_widget ("window_tenses");
+  gtk_window_set_title ((GtkWindow *) window, "English Tenses");
+  set_icon ((GtkWindow *) window);
+  g_signal_connect ((GObject *) window, "destroy",
+                    (GCallback) on_window_tenses_destroy, NULL);
+
+  combob_tenses = get_widget ("combob_tenses");
+  model = gtk_combo_box_get_model ((GtkComboBox *) combob_tenses);
+  gtk_list_store_clear ((GtkListStore *) model);
+  do {
+    gtk_combo_box_insert_text ((GtkComboBox *) combob_tenses, i, tenses[i]);
+  } while (++i < G_N_ELEMENTS (tenses));
+
+  textv_tenses = get_widget ("textv_tenses");
+  
+  g_signal_connect ((GObject *) combob_tenses, "changed",
+                    (GCallback) on_combob_tenses_changed, NULL);
+  gtk_combo_box_set_active ((GtkComboBox *) combob_tenses, 0);
+
+  button_tenses = get_widget ("button_tenses");
+  g_signal_connect ((GObject *) button_tenses, "clicked",
+                    (GCallback) on_window_tenses_destroy, NULL);
+
+  builder_destroy ();
+  
+  return window;
+}
+
 /* 
  * Buat GtkAboutDialog.
  */
@@ -573,7 +653,7 @@ create_dialog_about (void)
   /* set properti dialog_about */
   static const gchar copyright[] =
   (
-    "Copyright © 2008 - 2009, Ardhan Madras"
+    "Copyright \302\251 2008 - 2011 Ardhan Madras"
   );
 
   static const gchar license[] =
@@ -601,13 +681,11 @@ create_dialog_about (void)
   static const gchar *authors[] =
   {
     "Ardhan Madras <ajhwb@knac.com>",
-    "Firmansyah <leonard_gimsong@yahoo.com>",
     NULL
   };
 
   dialog = gtk_about_dialog_new ();
   set_icon ((GtkWindow *) dialog);
-  gtk_window_set_title ((GtkWindow *) dialog, g_strconcat ("Tentang", PROGRAM_NAME, NULL));
   gtk_about_dialog_set_name ((GtkAboutDialog *) dialog, PROGRAM_NAME);
   gtk_about_dialog_set_version ((GtkAboutDialog *) dialog, PROGRAM_VERS);
   gtk_about_dialog_set_copyright ((GtkAboutDialog *) dialog, copyright);
